@@ -5,22 +5,25 @@ Definition of :class:`EventList`.
 """
 
 import copy
+import pickle
+import warnings
+from collections.abc import Iterable
 
 import numpy as np
 import numpy.random as ra
+from astropy.table import Table
 
-from .io import read, write
-from .utils import simon, assign_value_if_none
+from .base import StingrayObject, StingrayTimeseries
 from .filters import get_deadtime_mask
-from .gti import cross_gtis, append_gtis, check_separate
-
+from .gti import append_gtis, check_separate, cross_gtis, generate_indices_of_boundaries
+from .io import load_events_and_gtis
 from .lightcurve import Lightcurve
-from stingray.simulator.base import simulate_times
+from .utils import assign_value_if_none, simon, interpret_times
 
 __all__ = ['EventList']
 
 
-class EventList(object):
+class EventList(StingrayTimeseries):
     """
     Basic class for event list data. Event lists generally correspond to individual events (e.g. photons)
     recorded by the detector, and their associated properties. For X-ray data where this type commonly occurs,
@@ -39,7 +42,7 @@ class EventList(object):
         to produce light curves with similar bin time.
 
     energy: iterable
-        A list of array of photon energy values
+        A list of array of photon energy values in keV
 
     mjdref : float
         The MJD used as a reference for the time array.
@@ -56,6 +59,34 @@ class EventList(object):
     notes : str
         Any useful annotations
 
+    high_precision : bool
+        Change the precision of self.time to float128. Useful while dealing with fast pulsars.
+
+    mission : str
+        Mission that recorded the data (e.g. NICER)
+
+    instr : str
+        Instrument onboard the mission
+
+    header : str
+        The full header of the original FITS file, if relevant
+
+    detector_id : iterable
+        The detector that recorded each photon (if the instrument has more than
+        one, e.g. XMM/EPIC-pn)
+
+    timeref : str
+        The time reference, as recorded in the FITS file (e.g. SOLARSYSTEM)
+
+    timesys : str
+        The time system, as recorded in the FITS file (e.g. TDB)
+
+    ephem : str
+        The JPL ephemeris used to barycenter the data, if any (e.g. DE430)
+
+    **other_kw :
+        Used internally. Any other keyword arguments will be ignored
+
     Attributes
     ----------
     time: numpy.ndarray
@@ -81,26 +112,60 @@ class EventList(object):
     pi : integer, numpy.ndarray
         PI channels
 
-    """
-    def __init__(self, time=None, energy=None, ncounts=None, mjdref=0, dt=0,
-                 notes="", gti=None, pi=None):
+    high_precision : bool
+        Change the precision of self.time to float128. Useful while dealing with fast pulsars.
 
-        self.energy = None if energy is None else np.array(energy)
+    mission : str
+        Mission that recorded the data (e.g. NICER)
+
+    instr : str
+        Instrument onboard the mission
+
+    detector_id : iterable
+        The detector that recoded each photon, if relevant (e.g. XMM, Chandra)
+
+    header : str
+        The full header of the original FITS file, if relevant
+
+    """
+    main_array_attr = "time"
+    def __init__(self, time=None, energy=None, ncounts=None, mjdref=0, dt=0,
+                 notes="", gti=None, pi=None, high_precision=False,
+                 mission=None, instr=None, header=None, detector_id=None,
+                 ephem=None, timeref=None, timesys=None,
+                 **other_kw):
+        StingrayObject.__init__(self)
+
+        self.energy = None if energy is None else np.asarray(energy)
         self.notes = notes
         self.dt = dt
         self.mjdref = mjdref
-        self.gti = gti
-        self.pi = pi
+        self.gti = np.asarray(gti) if gti is not None else None
+        self.pi = None if pi is None else np.asarray(pi)
         self.ncounts = ncounts
+        self.mission = mission
+        self.instr = instr
+        self.detector_id = detector_id
+        self.header = header
+        self.ephem = ephem
+        self.timeref = timeref
+        self.timesys = timesys
+
+        if other_kw != {}:
+            warnings.warn(f"Unrecognized keywords: {list(other_kw.keys())}")
 
         if time is not None:
-            self.time = np.array(time, dtype=np.longdouble)
-            self.ncounts = len(time)
+            time, mjdref = interpret_times(time, mjdref)
+            if not high_precision:
+                self.time = np.asarray(time)
+            else:
+                self.time = np.asarray(time, dtype=np.longdouble)
+            self.ncounts = self.time.size
         else:
             self.time = None
 
-        if (time is not None) and (energy is not None):
-            if len(time) != len(energy):
+        if (self.time is not None) and (self.energy is not None):
+            if self.time.size != self.energy.size:
                 raise ValueError('Lengths of time and energy must be equal.')
 
     def to_lc(self, dt, tstart=None, tseg=None):
@@ -124,7 +189,6 @@ class EventList(object):
         -------
         lc: :class:`stingray.Lightcurve` object
         """
-
         if tstart is None and self.gti is not None:
             tstart = self.gti[0][0]
             tseg = self.gti[-1][1] - tstart
@@ -133,8 +197,7 @@ class EventList(object):
                                           gti=self.gti, tseg=tseg,
                                           mjdref=self.mjdref)
 
-
-    def to_lc_list(self, dt):
+    def to_lc_iter(self, dt, segment_size=None):
         """Convert event list to a generator of Lightcurves.
 
         Parameters
@@ -142,25 +205,50 @@ class EventList(object):
         dt: float
             Binning time of the light curves
 
+        Other parameters
+        ----------------
+        segment_size : float, default None
+            Optional segment size. If None, use the GTI boundaries
+
         Returns
         -------
-        lc_gen: generator
-            Generates one :class:`stingray.Lightcurve` object for each GTI
+        lc_gen: `generator`
+            Generates one :class:`stingray.Lightcurve` object for each GTI or segment
         """
-        start_times = self.gti[:, 0]
-        end_times = self.gti[:, 1]
-        tsegs = end_times - start_times
 
-        for st, end, tseg in zip(start_times, end_times, tsegs):
-            idx_st = np.searchsorted(self.time, st, side='right')
-            idx_end = np.searchsorted(self.time, end, side='left')
-            lc = Lightcurve.make_lightcurve(self.time[idx_st:idx_end], dt,
+        segment_iter = generate_indices_of_boundaries(
+            self.time, self.gti, segment_size=segment_size, dt=0)
+
+        for st, end, idx_st, idx_end in segment_iter:
+            tseg = end - st
+
+            lc = Lightcurve.make_lightcurve(self.time[idx_st:idx_end + 1], dt,
                                             tstart=st,
-                                            gti=np.array([[st, end]]),
+                                            gti=np.asarray([[st, end]]),
                                             tseg=tseg,
-                                            mjdref=self.mjdref)
+                                            mjdref=self.mjdref, use_hist=True)
             yield lc
 
+    def to_lc_list(self, dt, segment_size=None):
+        """Convert event list to a list of Lightcurves.
+
+        Parameters
+        ----------
+        dt: float
+            Binning time of the light curves
+
+        Other parameters
+        ----------------
+        segment_size : float, default None
+            Optional segment size. If None, use the GTI boundaries
+
+        Returns
+        -------
+        lc_list: `List`
+            List containig one :class:`stingray.Lightcurve` object for each GTI or segment
+        """
+        return list(self.to_lc_iter(dt, segment_size))
+
     @staticmethod
     def from_lc(lc):
         """
@@ -186,9 +274,14 @@ class EventList(object):
         return EventList(time=times, gti=lc.gti)
 
     def simulate_times(self, lc, use_spline=False, bin_time=None):
-        """
-        Randomly assign (simulate) photon arrival times to an :class:`EventList` from a
-        :class:`stingray.Lightcurve` object, using the acceptance-rejection method.
+        """Simulate times from an input light curve.
+
+        Randomly simulate photon arrival times to an :class:`EventList` from a
+        :class:`stingray.Lightcurve` object, using the inverse CDF method.
+
+        ..note::
+            Preferably use model light curves containing **no Poisson noise**,
+            as this method will intrinsically add Poisson noise to them.
 
         Parameters
         ----------
@@ -199,31 +292,41 @@ class EventList(object):
         use_spline : bool
             Approximate the light curve with a spline to avoid binning effects
 
-        bin_time : float
-            The bin time of the light curve, if it needs to be specified for
-            improved precision
+        bin_time : float default None
+            Ignored and deprecated, maintained for backwards compatibility.
 
         Returns
         -------
         times : array-like
             Simulated photon arrival times
         """
-        self.time = simulate_times(lc, use_spline=use_spline,
-                                   bin_time=bin_time)
+        # Need import here, or there will be a circular import
+        from .simulator.base import simulate_times
+        if bin_time is not None:
+            warnings.warn("Bin time will be ignored in simulate_times",
+                          DeprecationWarning)
+
+        self.time = simulate_times(lc, use_spline=use_spline)
         self.gti = lc.gti
         self.ncounts = len(self.time)
 
-    def simulate_energies(self, spectrum):
+    def simulate_energies(self, spectrum, use_spline=False):
         """
         Assign (simulate) energies to event list from a spectrum.
 
         Parameters
         ----------
-        spectrum: 2-d array or list
+        spectrum: 2-d array or list [energies, spectrum]
             Energies versus corresponding fluxes. The 2-d array or list must
             have energies across the first dimension and fluxes across the
-            second one.
+            second one. If the dimension of the energies is the same as
+            spectrum, they are interpreted as bin centers.
+            If it is longer by one, they are interpreted as proper bin edges
+            (similarly to the bins of `np.histogram`).
+            Note that for non-uniformly binned spectra, it is advisable to pass
+            the exact edges.
         """
+        from .simulator.base import simulate_with_inverse_cdf
 
         if self.ncounts is None:
             simon("Either set time values or explicity provide counts.")
@@ -231,8 +334,8 @@ class EventList(object):
 
         if isinstance(spectrum, list) or isinstance(spectrum, np.ndarray):
 
-            energy = np.array(spectrum)[0]
-            fluxes = np.array(spectrum)[1]
+            energy = np.asarray(spectrum)[0]
+            fluxes = np.asarray(spectrum)[1]
 
             if not isinstance(energy, np.ndarray):
                 raise IndexError("Spectrum must be a 2-d array or list")
@@ -240,21 +343,55 @@ class EventList(object):
         else:
             raise TypeError("Spectrum must be a 2-d array or list")
 
-        # Create a set of probability values
-        prob = fluxes / float(sum(fluxes))
+        if energy.size == fluxes.size:
+            de = energy[1] - energy[0]
+            energy = np.concatenate([energy - de / 2, [energy[-1] + de / 2]])
 
-        # Calculate cumulative probability
-        cum_prob = np.cumsum(prob)
+        self.energy = simulate_with_inverse_cdf(
+            fluxes, self.ncounts, edges=energy, sorted=False, interp_kind="linear")
 
-        # Draw N random numbers between 0 and 1, where N is the size of event
-        # list
-        R = ra.uniform(0, 1, self.ncounts)
+    def sort(self, inplace=False):
+        """Sort the event list in time.
 
-        # Assign energies to events corresponding to the random numbers drawn
-        self.energy = \
-            np.array([energy[
-                np.argwhere(cum_prob == np.min(cum_prob[(cum_prob - r) > 0]))]
-                      for r in R])
+        Other parameters
+        ----------------
+        inplace : bool, default False
+            Sort in place. If False, return a new event list.
+
+        Returns
+        -------
+        eventlist : `EventList`
+            The sorted event list. If ``inplace=True``, it will be a shallow copy
+            of ``self``.
+
+        Examples
+        --------
+        >>> events = EventList(time=[0, 2, 1], energy=[0.3, 2, 0.5], pi=[3, 20, 5])
+        >>> e1 = events.sort()
+        >>> np.allclose(e1.time, [0, 1, 2])
+        True
+        >>> np.allclose(e1.energy, [0.3, 0.5, 2])
+        True
+        >>> np.allclose(e1.pi, [3, 5, 20])
+        True
+
+        But the original event list has not been altered (``inplace=False`` by
+        default):
+        >>> np.allclose(events.time, [0, 2, 1])
+        True
+
+        Let's do it in place instead
+        >>> e2 = events.sort(inplace=True)
+        >>> np.allclose(e2.time, [0, 1, 2])
+        True
+
+        In this case, the original event list has been altered.
+        >>> np.allclose(events.time, [0, 1, 2])
+        True
+
+        """
+        order = np.argsort(self.time)
+        return self.apply_mask(order, inplace=inplace)
 
     def join(self, other):
         """
@@ -350,103 +487,167 @@ class EventList(object):
             else:
                 ev_new.gti = cross_gtis([self.gti, other.gti])
 
+        for attr in ['mission', 'instr']:
+            if getattr(self, attr) != getattr(other, attr):
+                setattr(ev_new, attr, getattr(self, attr) + ',' + getattr(other, attr))
+            else:
+                setattr(ev_new, attr, getattr(self, attr))
+
         ev_new.mjdref = self.mjdref
 
         return ev_new
 
-    @staticmethod
-    def read(filename, format_='pickle'):
-        """
-        Read an event list from a file on disk. The file must be either a Python pickle file (not recommended
-        for long-term storage), an HDF5 file, an ASCII or a FITS file. The file can have the following
-        attributes in the data or meta-data:
+    @classmethod
+    def read(cls, filename, fmt=None, format_=None, **kwargs):
+        r"""Read a :class:`EventList` object from file.
 
-        * ``time``:  the time stamps of the photon arrivals
-        * ``energy``: the photon energy corresponding to each time stamp
-        * ``ncounts``: the total number of photon counts recorded
-        * ``mjdref``: a reference time in Modified Julian Date
-        * ``dt``: the time resolution of the data
-        * ``notes``: other possible meta-data
-        * ``gti``: Good Time Intervals
-        * ``pi``: some instruments record energies as "Pulse Invariant", an integer number recorded from
-          the Pulse Height Amplitude
+        Currently supported formats are
+
+        * pickle (not recommended for long-term storage)
+        * hea or ogip : FITS Event files from (well, some) HEASARC-supported missions.
+        * any other formats compatible with the writers in
+          :class:`astropy.table.Table` (ascii.ecsv, hdf5, etc.)
+
+        Files that need the :class:`astropy.table.Table` interface MUST contain
+        at least a ``time`` column. Other recognized columns are ``energy`` and
+        ``pi``.
+        The default ascii format is enhanced CSV (ECSV). Data formats
+        supporting the serialization of metadata (such as ECSV and HDF5) can
+        contain all eventlist attributes such as ``mission``, ``gti``, etc with
+        no significant loss of information. Other file formats might lose part
+        of the metadata, so must be used with care.
 
         Parameters
         ----------
         filename: str
-            Name of the :class:`EventList` object to be read.
+            Path and file name for the file to be read.
 
-        format_: str
-            Available options are ``pickle``, ``hdf5``, ``ascii`` and `fits``.
+        fmt: str
+            Available options are 'pickle', 'hea', and any `Table`-supported
+            format such as 'hdf5', 'ascii.ecsv', etc.
+
+        Other parameters
+        ----------------
+        kwargs : dict
+            Any further keyword arguments to be passed to `load_events_and_gtis`
+            for reading in event lists in OGIP/HEASOFT format
 
         Returns
         -------
         ev: :class:`EventList` object
             The :class:`EventList` object reconstructed from file
         """
+        if fmt is None and format_ is not None:
+            warnings.warn(
+                "The format_ keyword for read and write is deprecated. "
+                "Use fmt instead", DeprecationWarning)
+            fmt = format_
 
-        attributes = ['time', 'energy', 'ncounts', 'mjdref', 'dt',
-                      'notes', 'gti', 'pi']
-        data = read(filename, format_, cols=attributes)
+        if fmt.lower() in ('hea', 'ogip'):
+            evtdata = load_events_and_gtis(filename, **kwargs)
 
-        if format_ == 'ascii':
-            time = np.array(data.columns[0])
-            return EventList(time=time)
+            evt = EventList(time=evtdata.ev_list,
+                            gti=evtdata.gti_list,
+                            pi=evtdata.pi_list,
+                            energy=evtdata.energy_list,
+                            mjdref=evtdata.mjdref,
+                            instr=evtdata.instr,
+                            mission=evtdata.mission,
+                            header=evtdata.header,
+                            detector_id=evtdata.detector_id,
+                            ephem=evtdata.ephem,
+                            timeref=evtdata.timeref,
+                            timesys=evtdata.timesys)
+            if 'additional_columns' in kwargs:
+                for key in evtdata.additional_data:
+                    if not hasattr(evt, key.lower()):
+                        setattr(evt, key.lower(), evtdata.additional_data[key])
+            return evt
 
-        elif format_ == 'hdf5' or format_ == 'fits':
-            keys = data.keys()
-            values = []
+        return super().read(filename=filename, fmt=fmt)
 
-            if format_ == 'fits':
-                attributes = [a.upper() for a in attributes]
-
-            for attribute in attributes:
-                if attribute in keys:
-                    values.append(data[attribute])
-
-                else:
-                    values.append(None)
-
-            return EventList(time=values[0], energy=values[1],
-                             ncounts=values[2], mjdref=values[3], dt=values[4],
-                             notes=values[5], gti=values[6], pi=values[7])
-
-        elif format_ == 'pickle':
-            return data
-
-        else:
-            raise KeyError("Format not understood.")
-
-    def write(self, filename, format_='pickle'):
-        """
-        Write an :class:`EventList` object to file. Possible file formats are ``pickle``, ``hdf5``, ``ascii``
-        or ``fits``.
+    def filter_energy_range(self, energy_range, inplace=False, use_pi=False):
+        """Filter the event list from a given energy range.
 
         Parameters
         ----------
-        filename: str
-            Name and path of the file to save the event list to..
+        energy_range: [float, float]
+            Energy range in keV, or in PI channel (if ``use_pi`` is True)
+
+        Other Parameters
+        ----------------
+        inplace : bool, default False
+            Do the change in place (modify current event list). Otherwise, copy
+            to a new event list.
+        use_pi : bool, default False
+            Use PI channel instead of energy in keV
+
+        Examples
+        --------
+        >>> events = EventList(time=[0, 1, 2], energy=[0.3, 0.5, 2], pi=[3, 5, 20])
+        >>> e1 = events.filter_energy_range([0, 1])
+        >>> np.allclose(e1.time, [0, 1])
+        True
+        >>> np.allclose(events.time, [0, 1, 2])
+        True
+        >>> e2 = events.filter_energy_range([0, 10], use_pi=True, inplace=True)
+        >>> np.allclose(e2.time, [0, 1])
+        True
+        >>> np.allclose(events.time, [0, 1])
+        True
 
-        format_: str
-            The file format to store the data in.
-            Available options are ``pickle``, ``hdf5``, ``ascii``, ``fits``
         """
-
-        if format_ == 'ascii':
-            write(np.array([self.time]).T, filename, format_, fmt=["%s"])
-
-        elif format_ == 'pickle':
-            write(self, filename, format_)
-
-        elif format_ == 'hdf5':
-            write(self, filename, format_)
-
-        elif format_ == 'fits':
-            write(self, filename, format_, tnames=['EVENTS', 'GTI'],
-                  colsassign={'gti': 'GTI'})
-
+        if use_pi:
+            energies = self.pi
         else:
-            raise KeyError("Format not understood.")
+            energies = self.energy
+        mask = (energies >= energy_range[0]) & (energies < energy_range[1])
+
+        return self.apply_mask(mask, inplace=inplace)
+
+    def apply_mask(self, mask, inplace=False):
+        """Apply a mask to all array attributes of the event list
+
+        Parameters
+        ----------
+        mask : array of ``bool``
+            The mask. Has to be of the same length as ``self.time``
+
+        Other parameters
+        ----------------
+        inplace : bool
+            If True, overwrite the current event list. Otherwise, return a new one.
+
+        Examples
+        --------
+        >>> evt = EventList(time=[0, 1, 2], mission="nustar")
+        >>> evt.bubuattr = [222, 111, 333]
+        >>> newev0 = evt.apply_mask([True, True, False], inplace=False);
+        >>> newev1 = evt.apply_mask([True, True, False], inplace=True);
+        >>> newev0.mission == "nustar"
+        True
+        >>> np.allclose(newev0.time, [0, 1])
+        True
+        >>> np.allclose(newev0.bubuattr, [222, 111])
+        True
+        >>> np.allclose(newev1.time, [0, 1])
+        True
+        >>> evt is newev1
+        True
+        """
+        array_attrs = self.array_attrs()
+
+        if inplace:
+            new_ev = self
+        else:
+            new_ev = EventList()
+            for attr in self.meta_attrs():
+                setattr(new_ev, attr, copy.deepcopy(getattr(self, attr)))
+
+        for attr in array_attrs:
+            if hasattr(self, attr) and getattr(self, attr) is not None:
+                setattr(new_ev, attr, copy.deepcopy(np.asarray(getattr(self, attr))[mask]))
+        return new_ev
 
     def apply_deadtime(self, deadtime, inplace=False, **kwargs):
         """Apply deadtime filter to this event list.
@@ -483,78 +684,29 @@ class EventList(object):
         >>> filt_events is events
         False
         >>> expected = np.array([1, 2, 2.2, 3, 3.2])
-        >>> np.all(filt_events.time == expected)
+        >>> np.allclose(filt_events.time, expected)
         True
-        >>> np.all(filt_events.pi == 1)
+        >>> np.allclose(filt_events.pi, 1)
         True
-        >>> np.all(filt_events.energy == 1)
+        >>> np.allclose(filt_events.energy, 1)
         True
-        >>> np.all(events.pi == 1)
+        >>> np.allclose(events.pi, 1)
         False
         >>> filt_events = events.apply_deadtime(0.11, inplace=True,
         ...                                     verbose=False)
         >>> filt_events is events
         True
         """
-        if inplace:
-            new_ev = self
-        else:
-            new_ev = copy.deepcopy(self)
-
         local_retall = kwargs.pop('return_all', False)
 
-        mask, retall = get_deadtime_mask(new_ev.time, deadtime,
+        mask, retall = get_deadtime_mask(self.time, deadtime,
                                          return_all=True,
                                          **kwargs)
 
-        for attr in 'time', 'energy', 'pi':
-            if hasattr(new_ev, attr):
-                setattr(new_ev, attr, getattr(new_ev, attr)[mask])
+        new_ev = self.apply_mask(mask, inplace=inplace)
 
         if local_retall:
             new_ev = [new_ev, retall]
 
         return new_ev
 
-    def change_mjdref(self, new_mjdref):
-        """Change the MJD reference time (MJDREF) of the light curve.
-
-        Times will be now referred to this new MJDREF
-
-        Parameters
-        ----------
-        new_mjdref : float
-            New MJDREF
-
-        Returns
-        -------
-        new_lc : :class:`EventList` object
-            The new LC shifted by MJDREF
-        """
-        time_shift = (self.mjdref - new_mjdref) * 86400
-
-        new_ev = self.shift(time_shift)
-        new_ev.mjdref = new_mjdref
-        return new_ev
-
-    def shift(self, time_shift):
-        """
-        Shift the events and the GTIs in time.
-
-        Parameters
-        ----------
-        time_shift: float
-            The time interval by which the light curve will be shifted (in
-            the same units as the time array in :class:`Lightcurve`
-
-        Returns
-        -------
-        new_ev : lightcurve.Lightcurve object
-            The new event list shifted by ``time_shift``
-
-        """
-        new_ev = copy.deepcopy(self)
-        new_ev.time = new_ev.time + time_shift
-        new_ev.gti = new_ev.gti + time_shift
-
-        return new_ev
