@@ -5,9 +5,11 @@ Definition of :class::class:`Lightcurve`.
 or to save existing light curves in a class that's easy to use.
 """
 import os
+import copy
 import logging
 import warnings
 import pickle
+from collections.abc import Iterable
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -23,6 +25,7 @@ from stingray.gti import (bin_intervals_from_gtis, check_gtis, create_gti_mask,
 from stingray.utils import (assign_value_if_none, baseline_als,
                             poisson_symmetrical_errors, simon, interpret_times)
 from stingray.io import lcurve_from_fits
+from stingray import bexvar
 
 __all__ = ["Lightcurve"]
 
@@ -73,9 +76,25 @@ class Lightcurve(StingrayTimeseries):
         uncertainties and other statistical values appropriately.
         Default makes no assumptions and keep errors equal to zero.
 
+    bg_counts: iterable,`:class:numpy.array` or `:class:List` of floats, optional, default ``None``
+        A list or array of background counts detected in the background extraction region
+        in each bin corresponding to the bins defined in `time`.
+
+    bg_ratio: iterable, `:class:numpy.array` or `:class:List` of floats, optional, default ``None``
+        A list or array of source region area to background region area ratio in each bin. These are
+        factors by which the `bg_counts` should be scaled to estimate background counts within the 
+        source aperture.
+
+    frac_exp: iterable, `:class:numpy.array` or `:class:List` of floats, optional, default ``None``
+        A list or array of fractional exposers in each bin. 
+
     mjdref: float
         MJD reference (useful in most high-energy mission data)
 
+    dt: float or array of floats. Default median(diff(time))
+        Time resolution of the light curve. Can be an array of the same dimension
+        as ``time`` specifying width of each bin.
+
     skip_checks: bool
         If True, the user specifies that data are already sorted and contain no
         infinite or nan points. Use at your own risk
@@ -114,6 +133,15 @@ class Lightcurve(StingrayTimeseries):
     counts_err: numpy.ndarray
         The uncertainties corresponding to ``counts``
 
+    bg_counts: numpy.ndarray
+        The background counts corresponding to the bins in `time`.
+
+    bg_ratio: numpy.ndarray
+        The ratio of source region area to background region area corresponding to each bin.
+
+    frac_exp: numpy.ndarray
+        The fractional exposers in each bin.
+
     countrate: numpy.ndarray
         The counts per second in each of the bins defined in ``time``.
 
@@ -129,7 +157,7 @@ class Lightcurve(StingrayTimeseries):
     n: int
         The number of data points in the light curve.
 
-    dt: float
+    dt: float or array of floats
         The time resolution of the light curve.
 
     mjdref: float
@@ -167,9 +195,9 @@ class Lightcurve(StingrayTimeseries):
     """
     main_array_attr = "time"
 
-    def __init__(self, time, counts, err=None, input_counts=True,
-                 gti=None, err_dist='poisson', mjdref=0, dt=None,
-                 skip_checks=False, low_memory=False, mission=None,
+    def __init__(self, time, counts, err=None, input_counts=True, gti=None,
+                 err_dist='poisson', bg_counts=None, bg_ratio=None, frac_exp=None,
+                 mjdref=0, dt=None, skip_checks=False, low_memory=False, mission=None,
                  instr=None, header=None, **other_kw):
 
         StingrayTimeseries.__init__(self)
@@ -223,10 +251,18 @@ class Lightcurve(StingrayTimeseries):
             dt = np.median(np.diff(self._time))
 
         self.dt = dt
+
+        if isinstance(dt, Iterable):
+            warnings.warn("Some functionalities of Stingray Lightcurve will not work when `dt` is Iterable")
+
         self.err_dist = err_dist
 
-        self.tstart = self._time[0] - 0.5 * self.dt
-        self.tseg = self._time[-1] - self._time[0] + self.dt
+        if isinstance(self.dt, Iterable):
+            self.tstart = self._time[0] - 0.5 * self.dt[0]
+            self.tseg = self._time[-1] - self._time[0] + self.dt[-1] / 2 + self.dt[0] / 2
+        else:
+            self.tstart = self._time[0] - 0.5 * self.dt
+            self.tseg = self._time[-1] - self._time[0] + self.dt
 
         self._gti = None
         if gti is not None:
@@ -255,6 +291,19 @@ class Lightcurve(StingrayTimeseries):
             self._countrate = np.asarray(counts)
             self._countrate_err = err
 
+        if bg_counts is not None:
+            self.bg_counts = np.asarray(bg_counts)
+        else:
+            self.bg_counts = None
+        if bg_ratio is not None:
+            self.bg_ratio = np.asarray(bg_ratio)
+        else:
+            self.bg_ratio = None
+        if frac_exp is not None:
+            self.frac_exp = np.asarray(frac_exp)
+        else:
+            self.frac_exp = None
+
         if not skip_checks:
             self.check_lightcurve()
         if os.name == 'nt':
@@ -394,7 +443,7 @@ class Lightcurve(StingrayTimeseries):
         if countrate_err is None and self._counts_err is not None:
             countrate_err = self._counts_err / self.dt
         elif countrate_err is None:
-            countrate_err = 0
+            countrate_err = np.zeros(np.size(self.time))
 
         # If not in low-memory regime, cache the values ONLY if they have
         # been changed!
@@ -444,12 +493,7 @@ class Lightcurve(StingrayTimeseries):
         unsorted = np.any(dt_array < 0)
 
         if unsorted:
-            logging.warning("The light curve is unsorted. Now, sorting...")
-            order = np.argsort(time)
-            time = time[order]
-            counts = counts[order]
-            if err is not None:
-                err = err[order]
+            logging.warning("The light curve is unsorted.")
         return time, counts, err
 
     def check_lightcurve(self):
@@ -464,15 +508,17 @@ class Lightcurve(StingrayTimeseries):
         check_gtis(self.gti)
 
         idxs = np.searchsorted(self.time, self.gti)
-        uneven = False
-        for idx in range(idxs.shape[0]):
-            istart, istop = idxs[idx, 0], min(idxs[idx, 1], self.time.size - 1)
+        uneven = isinstance(self.dt, Iterable)
 
-            local_diff = np.diff(self.time[istart:istop])
-            if np.any(~np.isclose(local_diff, self.dt)):
-                uneven = True
+        if not uneven:
+            for idx in range(idxs.shape[0]):
+                istart, istop = idxs[idx, 0], min(idxs[idx, 1], self.time.size - 1)
 
-                break
+                local_diff = np.diff(self.time[istart:istop])
+                if np.any(~np.isclose(local_diff, self.dt)):
+                    uneven = True
+
+                    break
         if uneven:
             simon("Bin sizes in input time array aren't equal throughout! "
                   "This could cause problems with Fourier transforms. "
@@ -879,6 +925,10 @@ class Lightcurve(StingrayTimeseries):
 
         bin_time, bin_counts, bin_err = [], [], []
         gti_new = []
+
+        # If it does not exist, we create it on the spot
+        self.counts_err
+
         for g in self.gti:
             if g[1] - g[0] < dt_new:
                 continue
@@ -889,6 +939,7 @@ class Lightcurve(StingrayTimeseries):
 
                 t_temp = self.time[start_ind:end_ind]
                 c_temp = self.counts[start_ind:end_ind]
+
                 e_temp = self.counts_err[start_ind:end_ind]
 
                 bin_t, bin_c, bin_e, _ = \
@@ -1093,16 +1144,22 @@ class Lightcurve(StingrayTimeseries):
 
     def _truncate_by_index(self, start, stop):
         """Private method for truncation using index values."""
-        time_new = self.time[start:stop]
-        counts_new = self.counts[start:stop]
-        counts_err_new = self.counts_err[start:stop]
+
+        new_lc = self.apply_mask(slice(start, stop))
+
+        dtstart = dtstop = new_lc.dt
+        if isinstance(self.dt, Iterable):
+            dtstart = self.dt[0]
+            dtstop = self.dt[-1]
+
         gti = \
             cross_two_gtis(self.gti,
-                           np.asarray([[self.time[start] - 0.5 * self.dt,
-                                        time_new[-1] + 0.5 * self.dt]]))
+                           np.asarray([[new_lc.time[0] - 0.5 * dtstart,
+                                        new_lc.time[-1] + 0.5 * dtstop]]))
 
-        return Lightcurve(time_new, counts_new, err=counts_err_new, gti=gti,
-                          dt=self.dt, skip_checks=True)
+        new_lc.gti = gti
+
+        return new_lc
 
     def _truncate_by_time(self, start, stop):
         """Helper method for truncation using time values.
@@ -1134,6 +1191,24 @@ class Lightcurve(StingrayTimeseries):
 
         return self._truncate_by_index(start, stop)
 
+    def meta_attrs(self):
+        """Extends StingrayObject.meta_attrs to the specifics of Lightcurve."""
+        attrs = super().meta_attrs()
+        sure_array = ["counts", "counts_err", "countrate", "countrate_err"]
+        for attr in sure_array:
+            if attr in attrs:
+                attrs.remove(attr)
+        return attrs
+
+    def array_attrs(self):
+        """Extends StingrayObject.array_attrs to the specifics of Lightcurve."""
+        attrs = super().array_attrs()
+        sure_array = ["counts", "counts_err", "countrate", "countrate_err"]
+        for attr in sure_array:
+            if attr not in attrs:
+                attrs.append(attr)
+        return attrs
+
     def split(self, min_gap, min_points=1):
         """
         For data with gaps, it can sometimes be useful to be able to split
@@ -1187,7 +1262,7 @@ class Lightcurve(StingrayTimeseries):
         lc_split = self.split_by_gti(gti, min_points=min_points)
         return lc_split
 
-    def sort(self, reverse=False):
+    def sort(self, reverse=False, inplace=False):
         """
         Sort a Lightcurve object by time.
 
@@ -1199,6 +1274,8 @@ class Lightcurve(StingrayTimeseries):
         ----------
         reverse : boolean, default False
             If True then the object is sorted in reverse order.
+        inplace : bool
+            If True, overwrite the current light curve. Otherwise, return a new one.
 
         Examples
         --------
@@ -1217,20 +1294,13 @@ class Lightcurve(StingrayTimeseries):
             The :class:`Lightcurve` object with sorted time and counts
             arrays.
         """
-        new_time, new_counts, new_counts_err = \
-            zip(*sorted(zip(self.time, self.counts, self.counts_err),
-                        reverse=reverse))
-        new_time = np.asarray(new_time)
-        new_counts = np.asarray(new_counts)
-        new_counts_err = np.asarray(new_counts_err)
 
-        new_lc = Lightcurve(new_time, new_counts, err=new_counts_err,
-                            gti=self.gti, dt=self.dt, mjdref=self.mjdref,
-                            skip_checks=True)
+        mask = np.argsort(self.time)
+        if reverse:
+            mask = mask[::-1]
+        return self.apply_mask(mask, inplace=inplace)
 
-        return new_lc
-
-    def sort_counts(self, reverse=False):
+    def sort_counts(self, reverse=False, inplace=False):
         """
         Sort a :class:`Lightcurve` object in accordance with its counts array.
 
@@ -1242,6 +1312,8 @@ class Lightcurve(StingrayTimeseries):
         ----------
         reverse : boolean, default ``False``
             If ``True`` then the object is sorted in reverse order.
+        inplace : bool
+            If True, overwrite the current light curve. Otherwise, return a new one.
 
         Returns
         -------
@@ -1261,15 +1333,10 @@ class Lightcurve(StingrayTimeseries):
         True
         """
 
-        new_counts, new_time, new_counts_err = \
-            zip(*sorted(zip(self.counts, self.time, self.counts_err),
-                        reverse=reverse))
-
-        new_lc = Lightcurve(new_time, new_counts, err=new_counts_err,
-                            gti=self.gti, dt=self.dt, mjdref=self.mjdref,
-                            skip_checks=True)
-
-        return new_lc
+        mask = np.argsort(self.counts)
+        if reverse:
+            mask = mask[::-1]
+        return self.apply_mask(mask, inplace=inplace)
 
     def estimate_chunk_length(self, *args, **kwargs):
         """Deprecated alias of estimate_segment_size.
@@ -1686,39 +1753,115 @@ class Lightcurve(StingrayTimeseries):
             if (stop - start) < min_points:
                 continue
 
+            new_gti = np.array([gti[i]])
+            mask = create_gti_mask(self.time, new_gti)
+
             # Note: GTIs are consistent with default in this case!
-            new_lc = Lightcurve(self.time[start:stop], self.counts[start:stop],
-                                err=self.counts_err[start:stop],
-                                mjdref=self.mjdref, gti=[gti[i]],
-                                dt=self.dt, err_dist=self.err_dist,
-                                skip_checks=True)
+            new_lc = self.apply_mask(mask)
+            new_lc.gti = new_gti
+
             list_of_lcs.append(new_lc)
 
         return list_of_lcs
 
-    def apply_gtis(self):
+    def apply_mask(self, mask, inplace=False):
+        """Apply a mask to all array attributes of the event list
+
+        Parameters
+        ----------
+        mask : array of ``bool``
+            The mask. Has to be of the same length as ``self.time``
+
+        Other parameters
+        ----------------
+        inplace : bool
+            If True, overwrite the current light curve. Otherwise, return a new one.
+
+        Examples
+        --------
+        >>> lc = Lightcurve(time=[0, 1, 2], counts=[2, 3, 4], mission="nustar")
+        >>> lc.bubuattr = [222, 111, 333]
+        >>> newlc0 = lc.apply_mask([True, True, False], inplace=False);
+        >>> newlc1 = lc.apply_mask([True, True, False], inplace=True);
+        >>> newlc0.mission == "nustar"
+        True
+        >>> np.allclose(newlc0.time, [0, 1])
+        True
+        >>> np.allclose(newlc0.bubuattr, [222, 111])
+        True
+        >>> np.allclose(newlc1.time, [0, 1])
+        True
+        >>> lc is newlc1
+        True
+        """
+        array_attrs = self.array_attrs()
+
+        self._mask = self._n = None
+        if inplace:
+            new_ev = self
+        else:
+            new_ev = Lightcurve(time=self.time[mask], counts=self.counts[mask], skip_checks=True, gti=self.gti)
+            for attr in self.meta_attrs():
+                try:
+                    setattr(new_ev, attr, copy.deepcopy(getattr(self, attr)))
+                except AttributeError:
+                    continue
+
+        for attr in array_attrs:
+            if hasattr(self, "_" + attr) or attr in ["time", "counts"]:
+                continue
+            if hasattr(self, attr) and getattr(self, attr) is not None:
+                setattr(new_ev, attr, copy.deepcopy(np.asarray(getattr(self, attr))[mask]))
+        return new_ev
+
+    def apply_gtis(self, inplace=True):
         """
         Apply GTIs to a light curve. Filters the ``time``, ``counts``,
         ``countrate``, ``counts_err`` and ``countrate_err`` arrays for all bins
         that fall into Good Time Intervals and recalculates mean countrate
         and the number of bins.
+
+        Parameters
+        ----------
+        inplace : bool
+            If True, overwrite the current light curve. Otherwise, return a new one.
+
         """
+
         check_gtis(self.gti)
 
         good = self.mask
+        newlc = self.apply_mask(good, inplace=inplace)
+        dt = newlc.dt
+        if "dt" in self.array_attrs():
+            dt = newlc.dt[0]
+        newlc.tstart = newlc.time - 0.5 * dt
+        newlc.tseg = np.max(newlc.gti) - np.min(newlc.gti)
+        return newlc
 
-        # nota bene: We set the private properties, otherwise we'll get a
-        # ValueError from changing the shape of the arrays.
-        self._time = self.time[good]
-        self._counts = self.counts[good]
-        if self._counts_err is not None:
-            self._counts_err = self._counts_err[good]
-        self._countrate = None
-        self._countrate_err = None
-        self._mask = None
+    def bexvar(self):
+        """
+        Finds posterior samples of Bayesian excess varience (bexvar) for the light curve.
+        It requires source counts in ``counts`` and time intervals for each bin.
+        If the ``dt`` is an array then uses its elements as time intervals
+        for each bin. If ``dt`` is float, it calculates the time intervals by assuming
+        all intervals to be equal to ``dt``.
 
-        self._meanrate = None
-        self._meancounts = None
-        self._n = None
-        self.tseg = np.max(self.gti) - np.min(self.gti)
-        self.tstart = self.time - 0.5 * self.dt
+        Returns
+        -------
+        lc_bexvar : iterable, `:class:numpy.array` of floats
+            An array of posterior samples of Bayesian excess varience (bexvar).
+        """
+
+        # calculate time intervals for each bin if not provided by user
+        # assumes that time intervals in each bin are equal to ``dt``
+        if not isinstance(self.dt, Iterable):
+            time_del = self.dt*np.ones(shape=self.n)
+        else:
+            time_del = self.dt
+
+        lc_bexvar = bexvar.bexvar(time=self._time, time_del=time_del,
+                                  src_counts=self.counts, bg_counts=self.bg_counts,
+                                  bg_ratio=self.bg_ratio, frac_exp=self.frac_exp)
+
+        return lc_bexvar
